


#include <math.h>
#include <inttypes.h>
#include "config.h"
#include "tft_master_1.h"
#include "tft_gfx.h"
#include <stdlib.h>  // need for rand function
#include <stdio.h>
#include <plib.h>
#include "touch_master.h"
#define	SYS_FREQ 40000000
#include "pt_cornell_1_2.h"

#include "notation_function.h"

// to use for cap charging time
#define NOP asm("nop");
// 1/2 microsec
#define wait20 NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;
// one microsec
#define wait40 wait20;wait20;

#define	WHOLE 0
#define	HALF 1
#define	QUARTER 2
#define	EIGHTH 3

static UINT8 system_state = 0;

#define STATE_START		0
#define STATE_MODE   	1
#define STATE_FREEPLAY	2
#define STATE_NOTATION_PRE 3
#define STATE_NOTATION_RUN 4

volatile int piano_start = 0 , piano_end = 0, freemode = 0, blink = 0;;


// thread control structs
static struct pt pt_ctmu, pt_music, pt_pattern, pt_drawkey, pt_time, pt_UI, pt_touch, pt_replay;

volatile UINT16 XMIN = 40, YMIN = 70, XMAX = 340, YMAX = 710;

// used in CTMU thread for pin scanning 
volatile int pattern=0,key=0,key_num=0,s=0;
int key_save[40],press[40]; //store key number for which is pressed
int flag=0;  //trigger sound generation
int raw_adc0[40]; //store ADC value
int i=0,x=0,y=0; // variables for scanning

//Convert float to fix. a is a float
typedef signed int fix16 ;
#define float2fix16(a) ((fix16)((a)*65536.0)) // 2^16
#define fix2float16(a) ((float)(a)/65536.0)
// Fast fixed point multiply assembler macro
#define multfix16(a,b) ((fix16)(((( signed long long)(a))*(( signed long long)(b)))>>16)) //multiply two fixed 16:16

int generate_period = 2000 ; // PWM sample time
volatile int pwm_on_time = 1000 ;

// noise table 
volatile unsigned int p;
volatile signed int noise_table[256] ;

//variables used in KSA calculation 
//string 0
volatile signed int string[256], last_tune_out, last_tune_in, lowpass_out ;
signed int tune, damping ;
volatile char pluck, pushed ;
volatile int ptrin, ptrout;
//string 1
volatile signed int string1[256], last_tune_out1, last_tune_in1, lowpass_out1 ;
signed int tune1, damping1 ;
volatile char pluck1, pushed1 ;
volatile int ptrin1, ptrout1 ;
//string 2
volatile signed int string2[256], last_tune_out2, last_tune_in2, lowpass_out2 ;
signed int tune2, damping2 ;
volatile char pluck2, pushed2 ;
volatile int ptrin2, ptrout2;
//string 3
volatile signed int string3[256], last_tune_out3, last_tune_in3, lowpass_out3 ;
signed int tune3, damping3 ;
volatile char pluck3, pushed3 ;
volatile int ptrin3, ptrout3 ;

//used in music thread
char buffer[60]; 
volatile int drawkey=0,dtemp=0;
// string length 
int note[45]={153,144,136,129,121,115,108,102,96, 91, 86, 81,
              76, 72, 68, 64, 61, 57, 54, 51, 48, 45, 43, 40,
              38, 36, 34, 32, 30, 28, 27, 25, 24, 22, 21, 20,
              19, 32, 19, 32, 204};
volatile char key_name[28][4]={"C4", "C4#","D4","D4#","E4","F4", "F4#","G4","G4#","A4","A4#", "B4",
                               "C5", "C5#","D5","D5#","E5","F5", "F5#","G5","G5#","A5","A5#", "B5",
                               "C6",  "N",  "N", "D5#"};
int note_last=0;
int string_length[4]={122,122,122,122};// tune the string by it's length 
int str_en[4]={0,0,0,0};  //string enable

volatile int get_chord = 3, last_chord =50,cont=1;
#define C_chord 0
#define G_chord 2
#define NONE_chord 13
volatile char chord_name[3][10]={"C_chord", "G_chord","None"};

volatile int time_tick=0, notation_start=0, notation_run=0, notation_end=0;

volatile int dis[20], pit[20],a=0,b=0, dis_c[100], pit_c[100];

void ADC_init();




//interrupt service routine for KSA sound generation, sample time is 20kHz
void __ISR(_TIMER_2_VECTOR, ipl2) Timer2Handler(void) 
{
    mT2ClearIntFlag(); //clear flag
    SetDCOC2PWM(pwm_on_time); // PWM output
    //string 0
	// low pass filter
	lowpass_out = multfix16(damping, (string[ptrin] + string[ptrout])>>1) ;
 	// tuning all-pass filter
	string[ptrin] = multfix16(tune,(lowpass_out - last_tune_out)) + last_tune_in ;
    // all-pass state vars
    last_tune_out = string[ptrin];
    last_tune_in = lowpass_out;
	// update and wrap pointers
    if (ptrin==string_length[0]) ptrin=1;
    else ptrin=ptrin+1;
    if (ptrout==string_length[0]) ptrout=1; 
    else ptrout=ptrout+1;  
    
    //string 1
	// low pass filter
    lowpass_out1 = multfix16(damping1, (string1[ptrin1] + string1[ptrout1])>>1) ;
 	// tuning all-pass filter
	string1[ptrin1] = multfix16(tune1,(lowpass_out1 - last_tune_out1)) + last_tune_in1 ;
    // all-pass state vars
    last_tune_out1 = string1[ptrin1];
    last_tune_in1 = lowpass_out1;
    // update and wrap pointers
    if (ptrin1==string_length[1]) ptrin1=1;
    else ptrin1=ptrin1+1;	
    if (ptrout1==string_length[1]) ptrout1=1; 
    else ptrout1=ptrout1+1;  
    
    //string 2
	// low pass filter
    lowpass_out2 = multfix16(damping2, (string2[ptrin2] + string2[ptrout2])>>1) ;
 	// tuning all-pass filter
	string2[ptrin2] = multfix16(tune2,(lowpass_out2 - last_tune_out2)) + last_tune_in2 ;
    // all-pass state vars
    last_tune_out2 = string2[ptrin2];
    last_tune_in2 = lowpass_out2;
    // update and wrap pointers
    if (ptrin2==string_length[2]) ptrin2=1;
    else ptrin2=ptrin2+1;
    if (ptrout2==string_length[2]) ptrout2=1; 
    else ptrout2=ptrout2+1;  
    
    //string 3
	// low pass filter
    lowpass_out3 = multfix16(damping3, (string3[ptrin3] + string3[ptrout3])>>1) ;
 	// tuning all-pass filter
	string3[ptrin3] = multfix16(tune3,(lowpass_out3 - last_tune_out3)) + last_tune_in3 ;
    // all-pass state vars
    last_tune_out3 = string3[ptrin3];
    last_tune_in3 = lowpass_out3;
    // update and wrap pointers
    if (ptrin3==string_length[3]) ptrin3=1;
    else ptrin3=ptrin3+1;
    if (ptrout3==string_length[3]) ptrout3=1; 
    else ptrout3=ptrout3+1;  
    
    // calculation trigger
    //init string 0 
    if(str_en[0]==1)
    {
        //string assignment 
        for (p=0; p<string_length[0]; p++) 
        {
            string[p] = noise_table[p] <<16 ;
        }
        //init pointer and string enable
        ptrout = 2; 
        ptrin = 1; 
        str_en[0]=0;
    }
        
    //init string 1 
    if(str_en[1]==1)
    {
        //string assignment 
        for (p=0; p<string_length[1]; p++) 
        {
         	string1[p] = noise_table[p] <<16 ;
        }   
        //init pointer and string enable
        ptrout1 = 2; 
        ptrin1 = 1;  
        str_en[1]=0;
    }
        
    //init string 2 
    if(str_en[2]==1)
    {
        //string assignment
        for (p=0; p<string_length[2]; p++) 
        {
        	string2[p] = noise_table[p] <<16 ;
        }   
        //init pointer and string enable
        ptrout2 = 2;  
        ptrin2 = 1;
        str_en[2]=0;
    }
    
    //init string 3 
    if(str_en[3]==1)
    {
        //string assignment
        for (p=0; p<string_length[3]; p++) 
        {
            string3[p] = noise_table[p] <<16 ;
        }
        //init pointer and string enable
        ptrout3 = 2;  
        ptrin3 = 1;  
        str_en[3]=0;
    }
	
    
    //assign value for  output
    pwm_on_time =  1000 + ((fix2float16(string[ptrin]))+
                           (fix2float16(string1[ptrin1]))+
                           (fix2float16(string2[ptrin2]))+
                           (fix2float16(string3[ptrin3])))*4 ; 
    
}

static PT_THREAD (protothread_touch(struct pt *pt))
{
    PT_BEGIN(pt);
	
		do{
			// yield time 5 milliseconds
			PT_YIELD_TIME_msec(5);
			touch_getPoint();

			if (touch_point.DONE && touch_point.Z) {
//				debug();
				if (touch_point.X >= XMIN && touch_point.X <= XMAX)
					touch_point.X = touch_point.X - XMIN;
				else
					touch_point.Z = 0;	// make it an invalid touch

				if (touch_point.Y >= YMIN && touch_point.Y <= YMAX)
					touch_point.Y = touch_point.Y - YMIN;
				else
					touch_point.Z = 0;	// make it an invalid touch
			}

			
		} while (touch_point.DONE==0 || touch_point.Z==0);
        //debug
        /*tft_fillRoundRect(0,10, 150, 40, 1, HX8357_BLACK);
                tft_setCursor(0, 10);
                tft_setTextColor(HX8357_YELLOW); tft_setTextSize(2);
                sprintf(buffer,"x:%d, y:%d",touch_point.X , touch_point.Y );
                tft_writeString(buffer);*/
    
    PT_EXIT(pt);
	PT_END(pt);
} // touch thread

int num_re = 1;
static PT_THREAD (protothread_replay(struct pt *pt))
{
    
    PT_BEGIN(pt);
	do{
		// yield time 5 milliseconds
		PT_YIELD_TIME_msec(14);
        
		if (time_tick == dis_c[num_re]) 
        {
			string_length[0] = note[pit_c[num_re]]; //assign length
            str_en[0]=1; 
            num_re++;
		}
        /*tft_fillRoundRect(0,100, 150, 40, 1, HX8357_BLACK);
        tft_setCursor(0, 100);
        tft_setTextColor(HX8357_YELLOW); tft_setTextSize(2);
        sprintf(buffer,"%d,%d",dis_c[num_re],time_tick );
        tft_writeString(buffer); */
        //PT_YIELD_TIME_msec(100);
        time_tick++;	
	} while (num_re != b+1);
        //debug
    
    PT_EXIT(pt);
	PT_END(pt);
} // touch thread

static PT_THREAD (protothread_UI(struct pt *pt))
{
    PT_BEGIN(pt);
    system_state = STATE_START;
    while(1)
    {
        PT_YIELD_TIME_msec(50);
        if(system_state == STATE_START)
        {
            //tft_fillScreen(HX8357_BLACK);
            tft_setTextColor(HX8357_BLACK);
            tft_setTextSize(3);
            sprintf(buffer, "PRESS TO START\n");
            
            tft_setCursor(115, HX8357_TFTHEIGHT/2);
            tft_writeString(buffer);
            
            touch_reset();
            do{
                touch_getPoint();
                delay_ms(10);
            } while (touch_point.DONE==0 || touch_point.Z==0);
            
            system_state = STATE_MODE;
            if(system_state = STATE_MODE)
            {
                tft_fillScreen(HX8357_WHITE);
                tft_setTextColor(HX8357_BLACK);
                tft_setTextSize(4);
                sprintf(buffer, "SELECT THE MODE:\n");
                UINT16 left = (HX8357_TFTWIDTH-strlen(buffer)*6*2)/2;
                tft_setCursor(left, 100);
                tft_writeString(buffer);
                sprintf(buffer, "Free    Notation\n");
                tft_setCursor(left, 180);
                tft_writeString(buffer);
                sprintf(buffer, "Mode      Mode\n");
                tft_setCursor(left, 210);
                tft_writeString(buffer);
            }
        }
        else if(system_state == STATE_MODE)
        {
            piano_end = 0;
            PT_SPAWN(pt, &pt_touch, protothread_touch(&pt_touch));
            if( touch_point.Y < 300 ){
                system_state = STATE_NOTATION_RUN ;
                drawwholestaff();
                time_tick = 0;
                notation_start = 1;
                /*system_state = STATE_NOTATION_PRE;
                tft_fillScreen(HX8357_BLACK);
                tft_setTextColor(HX8357_YELLOW);
                tft_setTextSize(4);
                sprintf(buffer, "SET THE TEMPO:\n");
                UINT16 left = (HX8357_TFTWIDTH-strlen(buffer)*6*2)/2;
                tft_setCursor(left, HX8357_TFTHEIGHT/3);
                tft_writeString(buffer); */          
            }
            else if( touch_point.Y > 300 ){
                system_state = STATE_FREEPLAY;
                tft_fillScreen(HX8357_WHITE);
                tft_setTextColor(HX8357_BLACK);
                tft_setTextSize(4);
                sprintf(buffer, "HAVE FUN!\n");
                UINT16 left = (HX8357_TFTWIDTH-strlen(buffer)*6*2)/2;
                tft_setCursor(left+30, HX8357_TFTHEIGHT/4);
                tft_writeString(buffer); 
            }        
        }
        else if(system_state == STATE_FREEPLAY){
            piano_start = 1;
            
            if(piano_end)
            {
                piano_start = 0 ;
                tft_fillScreen(HX8357_WHITE);
                tft_setTextColor(HX8357_BLACK);
                tft_setTextSize(4);
                sprintf(buffer, "SELECT THE MODE:\n");
                UINT16 left = (HX8357_TFTWIDTH-strlen(buffer)*6*2)/2;
                tft_setCursor(left, 100);
                tft_writeString(buffer);
                sprintf(buffer, "Free    Notation\n");
                tft_setCursor(left, 180);
                tft_writeString(buffer);
                sprintf(buffer, "Mode      Mode\n");
                tft_setCursor(left, 210);
                tft_writeString(buffer);
                system_state = STATE_MODE;
            }        
        }
            
        else if(system_state == STATE_NOTATION_PRE)
        {
            PT_SPAWN(pt, &pt_touch, protothread_touch(&pt_touch));
            if(touch_point.X >200 && touch_point.X < 250 && touch_point.Y > 500 && touch_point.Y < 600 ){
                system_state = STATE_NOTATION_RUN;
                drawwholestaff();
                time_tick = 0;
                notation_start = 1;
            }
            if(touch_point.X >0 && touch_point.X < 50 && touch_point.Y > 500 && touch_point.Y < 600 ){
                system_state = STATE_START;
            }
            
        }
        else if(system_state == STATE_NOTATION_RUN)
        {
            if(notation_end)
            {      
                //replay?
                PT_SPAWN(pt, &pt_touch, protothread_touch(&pt_touch));
                if(touch_point.X >0 && touch_point.X < 25 && touch_point.Y > 20 && touch_point.Y < 100 ){
                    b = 0;
                    notation_end = 0;
                    time_tick = 0;
                    drawwholestaff(); 
                    notation_start = 1;
                    num_re = 1 ;
                }
                else if(touch_point.X >50 && touch_point.X < 100 && touch_point.Y > 20 && touch_point.Y < 100 ){
                    //notation_end = 0;
                    /*system_state = STATE_START;*/
                    time_tick = 0;
                    num_re = 1 ;
                    PT_SPAWN(pt, &pt_replay, protothread_replay(&pt_replay));
                    
                }
                else if(touch_point.X >130 && touch_point.X < 150 && touch_point.Y > 20 && touch_point.Y < 100 ){
                    notation_end = 0;
                    tft_fillScreen(HX8357_WHITE);
                    tft_setTextColor(HX8357_BLACK);
                    tft_setTextSize(4);
                    sprintf(buffer, "SELECT THE MODE:\n");
                    UINT16 left = (HX8357_TFTWIDTH-strlen(buffer)*6*2)/2;
                    tft_setCursor(left, 100);
                    tft_writeString(buffer);
                    sprintf(buffer, "Free    Notation\n");
                    tft_setCursor(left, 180);
                    tft_writeString(buffer);
                    sprintf(buffer, "Mode      Mode\n");
                    tft_setCursor(left, 210);
                    tft_writeString(buffer);
                    system_state = STATE_MODE;                  
                }
                
            }//piano enable
            
            
        }
    }
    PT_END(pt);
}

// Thread for string length assignment & debounce & chord playing
static PT_THREAD (protothread_music(struct pt *pt))
{
    PT_BEGIN(pt);
    while(1)
    {
        PT_WAIT_UNTIL(pt, flag == 1 ); // wait for start
        flag = 0; // clear flag     
        //string assignment
        for(x=0;x<40;x++)
        {
            if(system_state == STATE_NOTATION_RUN)
            {
            if(x>11 && x != 13 && x != 15 && x != 18 && x != 20 && x != 22 && x != 25 && x != 27 && x != 30 && x != 32 && x != 34 && x != 37){
                
                //assign string0 and jump to string1
                if(key_save[x] == 1 && press[x]==1)
                {  
                    string_length[y] = note[x]; //assign length
                    str_en[y]=1;  //string enable
                    dtemp=x;
                    press[x]=0;   // debounce     
                    y++; //jump pointer
                    drawkey=1;
                    if(y == 3) y = 0;
               
                }
            }
            else{
                if(key_save[x] == 1 && press[x]==1)
                {
                    get_chord = x;
                    if(get_chord == last_chord) get_chord = NONE_chord; 
                    last_chord = get_chord;
                    
                /*tft_fillRoundRect(0,10, 150, 40, 1, HX8357_BLACK);
                tft_setCursor(0, 10);
                tft_setTextColor(HX8357_YELLOW); tft_setTextSize(2);
                sprintf(buffer,"%s",chord_name[last_chord] );
                tft_writeString(buffer); */
                }  
            }         
            }   
            if(system_state == STATE_FREEPLAY)
            {              
                //assign string0 and jump to string
                if(key_save[39] == 1 && press[39]==1)
                {
                    piano_end = 1;
                    
                }
                else if(key_save[x] == 1 && press[x]==1)
                {   
                    string_length[y] = note[x]; //assign length
                    str_en[y]=1;  //string enable
                    dtemp=x;
                    press[x]=0;   // debounce     
                    y++; //jump pointer
                    if(y == 3) y = 0;
                }
            }
        }
    }
    PT_END(pt);
}


short nx=30;
short cbase=0;
short line=0;
static PT_THREAD (protothread_drawkey(struct pt *pt))
{  
    PT_BEGIN(pt);
    cbase = 70;
    nx = 40;
    while(1)
    {
        PT_WAIT_UNTIL(pt, drawkey == 1 ); // wait for start
        drawkey = 0; // clear flag
        nx = 40 + time_tick;//23;      
        smartnote(dtemp,nx,cbase,WHOLE);     
        a++;
        dis[a] = time_tick;
        pit[a] = dtemp;
        b++;
        dis_c[b] = time_tick+381*line;
        pit_c[b] = dtemp;
        /*tft_fillRoundRect(0,100, 150, 40, 1, HX8357_BLACK);
        tft_setCursor(0, 100);
        tft_setTextColor(HX8357_YELLOW); tft_setTextSize(2);
        sprintf(buffer,"%d,%d",dis[a],a );
        tft_writeString(buffer); 
        PT_YIELD_TIME_msec(100);*/
    }
    PT_END(pt);
}

volatile int flip=0/*,first=0,d=0*/;
static PT_THREAD (protothread_time(struct pt *pt))
{
    PT_BEGIN(pt);
    while(1)
    {
        PT_YIELD_TIME_msec(14);
        if(notation_start)
        {
            if(time_tick%24    == 0 || time_tick == 0)
            {
                 
                if(flip == 0) {
                    tft_fillRoundRect(1,1, 8 ,8, 4, HX8357_RED);
                    flip = 1;      
                    string_length[3] = 6; //assign length
                    str_en[3]=1;  //string enable
                }
                else if(flip ==1){
                    tft_fillRoundRect(1,1, 8 ,8, 4, HX8357_YELLOW);
                    flip = 0; 
                    string_length[3] = 5; //assign length
                    str_en[3]=1;  //string enable
                }   
            }
            time_tick++;
            if(time_tick == 191)
            {
                notation_start = 0;
                time_tick = 0;
                notation_run = 1;
                piano_start = 1;
            }
        }
        if(notation_run)
        {
            
            
            
            if(line == 0 && time_tick == 381) {
                line++;
                cbase = 145;
                nx = 40;
                time_tick = 0;
            }    
            else if(line == 1 && time_tick == 381) {
                line++;
                cbase = 220;
                nx = 40;
                time_tick = 0;
            }  
            else if(line == 2 && time_tick == 381  ) {
                line++;
                cbase = 295;
                nx = 40;
                time_tick = 0;
            } 
            else if(line == 3 && time_tick == 381  ) {
                line=0;
                notation_run = 0;
                notation_end = 1;
                piano_start = 0;
                cbase = 70;
                nx = 40;
                flip = 0;
                
                
            }
            
            if(time_tick%24    == 0 || time_tick == 0)
            {
                //tick-tock
                if(flip == 0) {
                    tft_fillRoundRect(1,1, 8 ,8, 4, HX8357_RED);
                    flip = 1;      
                    string_length[3] = 6; //assign length
                    str_en[3]=1;  //string enable
                }
                else if(flip ==1){
                    tft_fillRoundRect(1,1, 8 ,8, 4, HX8357_YELLOW);
                    flip = 0; 
                    string_length[3] = 5; //assign length
                    str_en[3]=1;  //string enable
                }   
            }
            
            
           
            time_tick++;
            tft_drawPixel(time_tick+40,cbase-60 , HX8357_BLUE);
                       
            if(time_tick==180) {smartjudge2(dis, pit, a, cbase, 180);a=0;}
            if(time_tick==372) {smartjudge2(dis, pit, a, cbase, 372);a=0;}
            //if(time_tick==265) {smartjudge2(dis, pit, a, cbase, 265);a=0;}           
        }
    }
    PT_END(pt);
}

static PT_THREAD (protothread_pattern(struct pt *pt))
{
    PT_BEGIN(pt);
    while(1) {
        if(get_chord == C_chord)
        {
            string_length[3] = note[0]; //assign length
            str_en[3]=1;  //string enable 
            PT_YIELD_TIME_msec(650);
            string_length[3] = note[7]; //assign length
            str_en[3]=1;  //string enable 
            PT_YIELD_TIME_msec(650);
            string_length[3] = note[4]; //assign length
            str_en[3]=1;  //string enable 
            PT_YIELD_TIME_msec(650);
            string_length[3] = note[7]; //assign length
            str_en[3]=1;  //string enable  
            PT_YIELD_TIME_msec(650);
        }
        if(get_chord == G_chord) 
        {
            string_length[3] = note[40]; //assign length
            str_en[3]=1;  //string enable 
            PT_YIELD_TIME_msec(650);
            string_length[3] = note[7]; //assign length
            str_en[3]=1;  //string enable 
            PT_YIELD_TIME_msec(650);
            string_length[3] = note[2]; //assign length
            str_en[3]=1;  //string enable 
            PT_YIELD_TIME_msec(650);
            string_length[3] = note[7]; //assign length
            str_en[3]=1;  //string enable  
            PT_YIELD_TIME_msec(650);
        }
            PT_YIELD_TIME_msec(5);
   
    }
    PT_END(pt);
}

//  CTMU thread for capacitance measurement & pin scanning
static PT_THREAD (protothread_ctmu(struct pt *pt))
{
    PT_BEGIN(pt);   
    CTMUCONbits.ON = 1; // Turn on CTMU
    // choose a current level
    CTMUCONbits.IRNG = 3;
    // scanning and measurement
    while(1) {      
        //Outer loop
        PT_WAIT_UNTIL(pt, piano_start == 1 ); // wait for start
        for(i=0; i<5; i++)
        {    
            // change ADC channel after inner loop done
            if     (i==0)  SetChanADC10(  ADC_CH0_POS_SAMPLEA_AN9  ); 
            else if(i==1)  SetChanADC10(  ADC_CH0_POS_SAMPLEA_AN11 );
            else if(i==2)  SetChanADC10(  ADC_CH0_POS_SAMPLEA_AN0  );
            else if(i==3)  SetChanADC10(  ADC_CH0_POS_SAMPLEA_AN1  );
            else if(i==4)  SetChanADC10(  ADC_CH0_POS_SAMPLEA_AN5  );  
            //inner loop
            for(pattern=0; pattern<8; pattern++)
            {        
                AcquireADC10(); // start ADC sampling 
                key = pattern;  // record key number
                pattern <<= 2;  // shift for use
                mPORTASetBits(pattern); // set address pattern   
                
                // dischrge the cap
                CTMUCONbits.IDISSEN = 1; // start drain of circuit
                // wait for discharge   
                WriteTimer3(0);
                PT_YIELD_UNTIL(pt, ReadTimer3()>5000); // opt:2000  steady:4000 
                //stop discharging     
                CTMUCONbits.IDISSEN = 0; // End drain of circuit
                
                // start charging and wait 2 microsecs
                ConfigIntTimer2(T2_INT_OFF | T2_INT_PRIOR_2);  //disable ISR when charging
                CTMUCONbits.EDG1STAT = 1;
                wait40;wait40;
                // end charging
                CTMUCONbits.EDG1STAT = 0;
                ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2); // enable ISR after done
                
                // stop samping and start conversion                  
                ConvertADC10(); // end sampling & start conversion       
                // wait for complete
                while (!AD1CON1bits.DONE){}; // Wait for ADC conversion       
                // read the result of channel from the idle buffer        
                raw_adc0[8*i+key] =  ReadADC10(0) ; 
                
                //judgment using the results
                if(raw_adc0[8*i+key]<250) // touche220
                {            
                    key_save[8*i+key]=1;   //record key number in the array                        
                }
                else  if(raw_adc0[8*i+key]>300) // non-touched300
                {      
                    key_save[8*i+key]=0;   
                    press[8*i+key]=1;      // for debounce    
                }                   
                mPORTAClearBits(pattern);  // clear address pattern
                pattern >>= 2;             // shift for plus1
            
            }// inner loop end
        }// outer loop end         
        flag=1;  // music thread begins to work             
    } // NEVER exit while
   // END WHILE(1)
    PT_END(pt);
} // ctmu thread

/*static PT_THREAD (protothread_free(struct pt *pt))
{
    PT_BEGIN(pt);   
    // scanning and measurement
    while(1) {      
        //Outer loop
        PT_WAIT_UNTIL(pt, free_mode == 1 ); // wait for start
        if(blink)
        {
            for ()
            tft_fillCircle(short x0, short y0, short r, unsigned short color);
        }
        
    }
    PT_END(pt);
}*/
// === Main  ======================================================
void main(void) {
    
    SYSTEMConfigPerformance(PBCLK);
    ANSELA = 0; ANSELB = 0; CM1CON = 0; CM2CON = 0;
 
    //configuration for CTMU thread
    ADC_init();
    // open timer2 for discharge
    OpenTimer3(T3_ON  |  T3_PS_1_1 , 50000);
    
    
    //Configuration for KSA
    //Create table
     for (i=0; i<256; i++)
    {
        noise_table[i] =  (i<17)? i*7 : (i<241)? 128-i : i*7-1792  ; 
    }  	
    
    //string0 configuration
    ptrout = 2; // circular pointer 
	ptrin = 1; // circular pointer 
	last_tune_out = 0 ;
	last_tune_in = 0 ;
	tune = float2fix16(0.12) ; 
	damping = float2fix16(0.993) ; // must be 0.5<damping<=1.0
	pluck = 0; 
    
    //string0 configuration
    ptrout1 = 2; // circular pointer 
	ptrin1 = 1; // circular pointer 
	last_tune_out1 = 0 ;
	last_tune_in1 = 0 ;
	tune1 = float2fix16(0.12) ; 
	damping1 = float2fix16(0.993) ; // must be 0.5<damping<=1.0   
    
    //string2 configuration
    ptrout2 = 2; // circular pointer 
	ptrin2 = 1; // circular pointer 
	last_tune_out2 = 0 ;
	last_tune_in2 = 0 ;
	tune2 = float2fix16(0.12) ; 
	damping2 = float2fix16(0.993) ; // must be 0.5<damping<=1.0
    
    //string3 configuration
    ptrout3 = 2; // circular pointer 
	ptrin3 = 1; // circular pointer 
	last_tune_out3 = 0 ;
	last_tune_in3 = 0 ;
	//string_length[1] = 122 ;
	tune3 = float2fix16(0.12) ; //0.8
	damping3 = float2fix16(0.993) ; // must be 0.5<damping<=1.0
    
    // KSA basic configuration
    OpenOC2(OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE, pwm_on_time , pwm_on_time);
    PPSOutput(2, RPB5, OC2);
   	OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, generate_period);
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2); 
    mT2ClearIntFlag(); //  clear the interrupt flag 
    
    //config threads 
    PT_setup();
    INTEnableSystemMultiVectoredInt();
    PT_INIT(&pt_ctmu);
    PT_INIT(&pt_music);
    PT_INIT(&pt_drawkey);
    //PT_INIT(&pt_pattern);
    PT_INIT(&pt_time);
    PT_INIT(&pt_UI);
 
    //  config port
    PORTSetPinsDigitalOut(IOPORT_A, BIT_2);
    PORTSetPinsDigitalOut(IOPORT_A, BIT_3);
    PORTSetPinsDigitalOut(IOPORT_A, BIT_4);
    
    tft_init_hw();
    tft_begin();
    tft_fillScreen(HX8357_WHITE);
    tft_setRotation(3);
    touch_init();
    tft_drawBitmap(0, 0, image_data_piano, 480, 320,HX8357_BLACK) ; 
    

   // round-robin scheduler for threads
   while (1){
        PT_SCHEDULE(protothread_ctmu(&pt_ctmu));
        PT_SCHEDULE(protothread_music(&pt_music));
        PT_SCHEDULE(protothread_drawkey(&pt_drawkey));
       // PT_SCHEDULE(protothread_pattern(&pt_pattern));
        PT_SCHEDULE(protothread_time(&pt_time));
        PT_SCHEDULE(protothread_UI(&pt_UI));
    }
} // main
// === end  ======================================================




void ADC_init()
{
    // set up the ADC ///////////////////////////////////////
	CloseADC10();	// ensure the ADC is off before setting the configuration
    #define PARAM1  ADC_FORMAT_INTG16 | ADC_CLK_MANUAL | ADC_AUTO_SAMPLING_OFF //
	#define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF    
    #define PARAM3 ADC_CONV_CLK_PB | ADC_SAMPLE_TIME_15 | ADC_CONV_CLK_Tcy //ADC_SAMPLE_TIME_15| ADC_CONV_CLK_Tcy
	#define PARAM4	ENABLE_AN9_ANA | ENABLE_AN11_ANA | ENABLE_AN0_ANA  | ENABLE_AN1_ANA | ENABLE_AN5_ANA | ENABLE_AN2_ANA | ENABLE_AN3_ANA
	#define PARAM5	SKIP_SCAN_ALL
	SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN9 ); // configure to sample AN11
	OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using the parameters defined above
	EnableADC10(); // Enable the ADC
}